#  File ergm/R/control.ergm.R
#  Part of the statnet package, http://statnetproject.org
#
#  This software is distributed under the GPL-3 license.  It is free,
#  open source, and has the attribution requirements (GPL Section 7) in
#    http://statnetproject.org/attribution
#
# Copyright 2003 Mark S. Handcock, University of Washington
#                David R. Hunter, Penn State University
#                Carter T. Butts, University of California - Irvine
#                Steven M. Goodreau, University of Washington
#                Martina Morris, University of Washington
# Copyright 2007 The statnet Development Team
######################################################################
control.ergm<-function(prop.weights="default",prop.args=NULL,
                       nr.maxit=100, calc.mcmc.se=TRUE, hessian=FALSE,
                       compress=FALSE,
                       maxNumDyadTypes=10000, 
                       maxedges=20000,
                       maxchanges=1000000,
                       MPLEsamplesize=50000, 
                       MPLEtype=c("glm", "penalized"),
                       trace=0,
                       steplength=0.5,
                       drop=TRUE,
                       force.mcmc=FALSE,
                       check.degeneracy=TRUE,
                       mcmc.precision=0.05,
                       metric=c("Likelihood","raw"),
                       method=c("BFGS","Nelder-Mead"),
                       trustregion=20,
                       initial.loglik=NULL,
                       initial.network=NULL,
                       style=c("Newton-Raphson","Robbins-Monro","Stochastic-Approximation"),
                       phase1_n=NULL, initial_gain=NULL, 
                       nsubphases="maxit", niterations=NULL, phase3_n=NULL,
                       RobMon.phase1n_base=7,
                       RobMon.phase2n_base=7,
                       RobMon.phase2sub=4,
                       RobMon.init_gain=0.4,
                       RobMon.phase3n=500,
                       dyninterval=1000,
                       packagenames="ergm",
                       parallel=0,
                       returnMCMCstats=TRUE){
  control<-list()
  for(arg in names(formals(sys.function())))
    control[[arg]]<-get(arg)
  
  control$MPLEtype<-match.arg(MPLEtype)
  control$metric<-match.arg(metric)
  control$method<-match.arg(method)
  control$style<-match.arg(style)
  control$nsubphases<-match.arg(nsubphases)
  if(missing(trustregion) & control$style=="Stochastic-Approximation"){
   control$trustregion <- 0.5
  }

  control
}
